
	TITLE	'BOOT LOADER MODULE FOR CP/M 3.0'

	; DEFINE LOGICAL VALUES:
TRUE		EQU	-1
FALSE		EQU	NOT TRUE

	; DETERMINE IF FOR BANK SELECT OR NOT:
	
BANKED		EQU	TRUE		;<<<<<<<<<<<<<<<< FULL BLOWN VERSION

	; DEFINE PUBLIC LABELS:
	PUBLIC	?INIT,?LDCCP,?RLCCP,?TIME
	PUBLIC	OUT$BLOCKS
	PUBLIC	?SMSG			;NOTE A PUBLIC ROUTINE I ADDED FOR
					;SENDING SPEECH STRINGS TO TALKER

	; EXTERNALLY DEFINED ENTRY POINTS AND LABELS:
	EXTRN	?PMSG,?CONIN
	EXTRN	@CIVEC,@COVEC,@AIVEC,@AOVEC,@LOVEC
	EXTRN 	@CBNK,?BNKSL

	IF	BANKED
	EXTRN	BANKBUF		;128 BYTE BUFFER IN MOVE MODULE FOR USE
				; DURING COLD AND WARM BOOTS
	ENDIF

	EXTRN	@SEC,@MIN,@HOUR,@DATE	;FIELDS HOLDING CURRENT TIME AND DATE


	; INCLUDE Z-80 MACROS:
	MACLIB	Z80


	; SOME MISCELLANEOUS EQUATES:
BDOS		EQU	5
CR		EQU	13	;ASCII CARRIAGE RETURN
LF		EQU	10	;ASCII LINEFEED


; -----------  S100 Computers Serial I/O and CMOS RTC Boards PORT ASSIGNMENTS
;
BCTL	EQU	0A0H		; CHANNEL B CONTROL
ACTL	EQU	0A1H		; CHANNEL A CONTROL
BDTA	EQU	0A2H		; CHANNEL B DATA
ADTA	EQU	0A3H		; CHANNEL A DATA

CMOSPort EQU	70H		;Base Port for CMOS Chip on MSDOS Support Board

YROFS	EQU	0		;Years offset in TBUFFER
MNOFS	EQU	1		;Month offset
DTOFS	EQU	2		;Date offset



;
	; WE CAN DO INITIALIZATION FROM BANKED MEMORY (IF WE HAVE IT):
	IF	BANKED
	DSEG	; INIT DONE FROM BANKED MEMORY
	ELSE
	CSEG	; INIT TO BE DONE FROM COMMON MEMORY
	ENDIF


	;;;;; ?INIT
	; HARDWARE INITIALIZATION OTHER THAN CHARACTER AND DISK I/O:
?INIT:
	; ASSIGN CONSOLE INPUT AND OUTPUT TO CRT:
	LXI	H,8000H		;SIGNIFIES DEVICE 0
	SHLD	@CIVEC		;CONSOLE INPUT VECTOR
	SHLD	@COVEC		;CONSOLE OUTPUT VECTOR

	; ASSIGN PRINTER TO LPT:
	LXI	H,4000H		;SIGNIFIES DEVICE 1
	SHLD	@LOVEC		;LIST OUTPUT VECTOR

	; ASSIGN AUX TO CRT1:
	LXI	H,02000H	;SIGNIFIES DEVICE 2
	SHLD	@AIVEC		;AUXILLIARY INPUT VECTOR
	SHLD	@AOVEC		;AUXILLIARY OUTPUT VECTOR

	; PRINT THE SIGN-ON MESSAGE:
	LXI	H,SIGNON$MSG	;POINT TO IT
	JMP	?PMSG		;AND PRINT IT
;
	;;;; ?SMSG
	; ROUTINE OUTPUTS SPEECH STRING TO TALKER
	IF BANKED
	CSEG			;TO BE ON THE SAFE SIDE
	ENDIF

?SMSG:	MOV	A,M
	ORA	A
	JRZ	SMSG1
	MOV	C,A
	CALL	SPEAKOUT
	INX	H
	JR	?SMSG
SMSG1:	MVI	C,0DH		;MUST END STRING '0', SO END WITH CR FOR TALKER
;				;FALL THROUGH TO SPEAKOUT
;
SPEAKOUT:			;Sent output to Speech synthizer via Serial port
	PUSH	D		
	LXI	D,512		;Will try 512 times, then timeout
SPXXX:	IN	BCTL		;(A0), Is SCC TX Buffer empty
	ANI	04H
	JNZ	SENDS		;NZ if ready to recieve character
	DCX	D
	MOV	A,E
	ORA	D
	JNZ	SPXXX
	POP	D
	RET
SENDS:	POP	D
	MOV	A,C
	OUT	BDTA		;(A2), Send it
	RET
;
;
	;;;;; OUT$BLOCKS
	; ROUTINE OUTPUTS SPECIFIED # BYTES TO SPECIFIED OUTPUT PORTS:
	IF	BANKED
	CSEG			;WE WANT THIS ROUTINE IN COMMON MEMORY
	ENDIF
OUT$BLOCKS:
	MOV	A,M		;GET A BYTE FROM THE BLOCK
	ORA	A		;END OF OUTPUT BLOCK ?
	RZ			;THEN DONE!!
	MOV	B,A		;ELSE PUT # BYTES TO SEND OUT IN [B]
	INX	H		;POINT TO PORT TO SEND TO
	MOV	C,M		;GET IT TO [C]
	INX	H		;POINT TO 1ST BYTE OF BLOCK TO SEND OUT
	OUTIR			;Z-80 BLOCK OUTPUT
	JR	OUT$BLOCKS


	;;;;; ?LDCCP
	; THIS ROUTINE IS ENTERED TO LOAD THE CCP.COM FILE INTO THE TPA BANK
	;  AT SYSTEM COLD START:
?LDCCP:
	; SET UP THE FCB FOR THE FILE OPERATION:
	XRA	A		;ZERO EXTENT
	STA	CCP$FCB+15
	LXI	H,0		;START AT BEGINNING OF FILE
	SHLD	FCB$NR

	; TRY TO OPEN THE CCP.COM FILE:
	LXI	D,CCP$FCB	;POINT TO FCB
	CALL	OPEN		;ATTEMPT THE OPEN OPERATION
	INR	A		;WAS IT ON THE DISK ?
	JRNZ	CCP$FOUND	;YES -- GO LOAD IT

	; WE ARRIVE HERE WHEN CCP.COM FILE WASN'T FOUND:
	LXI	H,CCP$MSG	;REPORT THE ERROR
	CALL	?PMSG
	CALL	?CONIN		;GET A RESPONSE
	JR	?LDCCP		;AND TRY AGAIN

	; FILE WAS OPENED OK -- READ IT IN:
CCP$FOUND:
	LXI	D,0100H		;LOAD AT BOTTOM OF TPA
	CALL	SETDMA		;BY SETTING THE NEXT DMA ADDRESS
	LXI	D,128		;SET MULTI SECTOR I/O COUNT
	CALL	SETMULTI	; TO ALLOW UP TO 16K BYTES IN ONE OPERATION
	LXI	D,CCP$FCB	;POINT TO THE FCB
	CALL	READ		;AND READ THE CCP IN

	; FOLLOWING CODE FOR BANKED SYSTEMS -- MOVES CCP IMAGE TO BANK 2
	;  FOR LATER RELOADING AT WARM STARTS:
	IF	BANKED
	LXI	H,0100H		;GET CCP IMAGE FROM START OF TPA
	MVI	B,25		;TRANSFER 25 LOGICAL SECTORS
	LDA	@CBNK		;GET CURRENT BANK
	PUSH	PSW		;AND SAVE IT
LD$1:
	PUSH	B		;SAVE SECTOR COUNT
	MVI	A,1		;SELECT TPA BANK
	CALL	?BNKSL
	LXI	B,128		;TRANSFER 128 BYTES TO TEMPORARY BUFFER
	LXI	D,BANKBUF	;TEMPORARY BUFFER ADDR IN [DE]
	PUSH	H		;SAVE SOURCE ADDRESS
	PUSH	D		;AND DESTINATION
	PUSH	B		;AND COUNT
	LDIR			;BLOCK MOVE SECTOR TO TEMPORARY BUFFER
	MVI	A,2		;SELECT BANK TO SAVE CCP IN
	CALL	?BNKSL
	POP	B		;GET BACK COUNT
	POP	H		;LAST DESTINATION WILL BE NEW SOURCE ADDR
	POP	D		;LAST SOURCE WILL BE NEW DESTINATION
	LDIR			;BLOCK MOVE SECTOR FROM BUFFER TO ALTERNATE
				; BANK
	XCHG			;NEXT ADDR WILL BE NEW SOURCE ADDR
	POP	B		;GET BACK SECTOR COUNT
	DJNZ	LD$1		;DROP SECTOR COUNT AND LOOP TILL DONE...
	POP	PSW		;WHEN DONE -- RESTORE ORIGINAL BANK
	JMP	?BNKSL
	ELSE

	; IF NON-BANKED WE RETURN THROUGH HERE:
	RET
	ENDIF


	;;;;; ?RLCCP
	; ROUTINE RELOADS CCP IMAGE FROM BANK 2 IF BANKED SYSTEM OR FROM THE
	;  DISK IF NON-BANKED VERSION:
?RLCCP:
	IF	BANKED
	; FOLLOWING CODE FOR BANKED VERSION:
	LXI	H,0100H		;GET CCP IMAGE FROM START OF ALTERNATE BUFFER
	MVI	B,25		;TRANSFER 25 LOGICAL SECTORS
	LDA	@CBNK		;GET CURRENT BANK
	PUSH	PSW		;AND SAVE IT
RL$1:
	PUSH	B		;SAVE SECTOR COUNT
	MVI	A,2		;SELECT ALTERNATE BANK
	CALL	?BNKSL
	LXI	B,128		;TRANSFER 128 BYTES TO TEMPORARY BUFFER
	LXI	D,BANKBUF	;TEMPORARY BUFFER ADDR IN [DE]
	PUSH	H		;SAVE SOURCE ADDRESS
	PUSH	D		;AND DESTINATION
	PUSH	B		;AND COUNT
	LDIR			;BLOCK MOVE SECTOR TO TEMPORARY BUFFER
	MVI	A,1		;PUT CCP TO TPA BANK
	CALL	?BNKSL
	POP	B		;GET BACK COUNT
	POP	H		;LAST DESTINATION WILL BE NEW SOURCE ADDR
	POP	D		;LAST SOURCE WILL BE NEW DESTINATION
	LDIR			;BLOCK MOVE SECTOR FROM BUFFER TO TPA BANK
	XCHG			;NEXT ADDR WILL BE NEW SOURCE ADDR
	POP	B		;GET BACK SECTOR COUNT
	DJNZ	RL$1		;DROP SECTOR COUNT AND LOOP TILL DONE...
	POP	PSW		;GET BACK LAST CURRENT BANK #
	JMP	?BNKSL		;SELECT IT AND RETURN
	ELSE

	; FOLLOWING CODE IS FOR NON-BANKED VERSIONS:
	JMP	?LDCCP		;JUST DO LOAD AS THOUGH COLD BOOT
	ENDIF


	;;;;; ?TIME
	; ROUTINE SETS/GETS TIME For S100Computers MSDOS Board
	; Bytes in @HOUR, @MIN, @SEC are read froim the clock and stored in BCD format
	; @DATE is days since Jan 1 1978 -- This is how CPM stores dates

?TIME:	
	; SETTING OR GETTING TIME ?

	PUSH	B
	PUSH	D
	PUSH	H
	PUSHIX
	
	MOV	A,C
	ORA	A
	JZ	GETTM			;0 = The BIOS needs to update SCB (@DATE, @ HOURS etc) getting data from RTC
	JMP	SETTM			;0FFH = Bios has just updated the SCB, needs to update CMOS RTC chip to new values


					
;--------- SET TIME & DATE ----------------------------------------------------------------------------------------					

;	This routine first gets the CPM locations @HOUR, @MIN & @SEC (stored in BCD)
;	and updates the CMOS chip.  It then gets the CPM Julian days (stored in binary) 
;	since 1/1/2000 and updates the CMOS chip Days, Month, Years (in BCD values).
;	Note the origional DR CPM+ calculated days (up to 12/31/1999) from 1/1/1978.
;	We will start counting days since Y2000 (adding 1F63H to the DR value) 
;	HOWEVER THIS MEANS DAYS < Y2000 WILL BE INVALID

SETTM:	CALL	UpdInPR			;CHECK FOR UPDATE IN PROCESS
	JNC	RTC3A			;GO AROUND, IF CLOCK OPERATING
	CALL	InitilizationStatus

RTC3A:	MVI	E,-2			;-2 goes to 0 for PortInc2 
	CALL	PortInc2		;Seconds Address
	XRA	A
	LDA	@SEC			;Set seconds on CMOS RTC chip
	OUT	CMOSPort+1		;STORE Seconds in chip

	CALL	PortInc2		;Minutes Address
	LDA	@MIN
	OUT	CMOSPort+1		;STORE Minutes in chip
		
	CALL	PortInc2		;Minutes Address
	LDA	@HOUR
	OUT	CMOSPort+1		;STORE Minutes in chip

					;Now the hard part, get days since 1/1/1978
					;from CPM store and update CMOS chip

	LHLD	@DATE			;CPM will have total days since 1/1/1978 (in binary) here
	LXI	D,1F63H			;We need to subtract 1/1/1978 to 12/31/1999 days worth
	DSBC	D			;[HL] will contain total count of days since 1/1/2000

	LXI	D,TBUFFER		;We will store year,month,day here (Note, in binary format)
	PUSH	D
	POPIX				;IX also points to TBUFFER also
	
	MVI	A,0			;Scan through years from 1/1/2000
	MOV	B,A			;Save in B
	
DLOOP:	MOV	A,B			;Get current Year
	ANI	3			;Test for leap year
	LXI	D,365			;First get number of leap year days
	JNZ	DNL1
	INX	D	
DNL1:	INR	B			;Increment year count (2000,2001,2002....)
	ORA	A			;Clear carry for DBL SUB
	DSBC	D			;<<<< Subtract DE from HL >>> until negative
	JZ	DENDY
	JNC	DLOOP			;Continue until we overshoot
	
DENDY:	DAD	D			;Overshot correct days
	DCR	B			;and year
	MOV	A,B			;Fetch calculated year
	CPI	100			;Rewind every century
	JC	DPYR			
	MVI	B,0			;Rewind year 100 to 0
DPYR:	STX	YROFS,B			;<---- Store Year in TBUFFER as Binary number

	MOV	A,B			;check leap year again
	ANI	3
	MVI	A,28			;28 days for leap year
	JNZ	DNL2
	INR	A			;Adjust for leap year
DNL2:	STA	MFEB			;Fix Feb value in month table
	LXI	D,MTBL			;Point to month table
	MVI	B,0			;Initial month
	
					;Scan through current year subtracting days
DMNTH:	PUSH	D			;Save month table pointer
	LDAX	D			;Get days in current month
	MOV	E,A			;Get in DE as 16 Bits
	MVI	D,0
	ORA	A			;Clear carry for DSBC
	DSBC	D
	JC	DFND			;Exit loop when overshoot
	JZ	DFND			;Last day of month
	INR	B			;Bump to next month
	POP	D			;Retrive month pointer
	INX	D			;Bump to next month
	JMP	DMNTH			;Continue loop
	
DFND:	POP	PSW			;Clear stack
	DAD	D			;Fix overshoot
	INR	B			;Make Month 1-Relative
	LXIX	TBUFFER			
	STX	B,MNOFS			;Store in TBUFFER
	STX	L,DTOFS
					;We now have (in binary) in TBUFFER year,month,day
					;Next send to chip
	LXI	H,TBUFFER+2
	MVI	E,5			;SET TO 0 ADDRESS OF DAY OF WEEK REGISTER (6)
	CALL	PortInc			;Just in case it is mis-read
	MVI	A,0H
	OUT	CMOSPort+1		;LOAD ZEROS TO 'DAY OF WEEK' BYTE
	
	MOV	A,M			;GET DAY OF MONTH BYTE
	CALL	BINTOBCD		;Convert [A] to two BCD digits	
	PUSH	PSW
	CALL	PortInc			;ADDRESS OF DAY OF MONTH REGISTER (7)
	POP	PSW
	OUT	CMOSPort+1		;(Quickly) STORE IT
	
	DCX	H
	MOV	A,M			;GET MONTH BYTE
	CALL	BINTOBCD		;Convert [A] to two BCD digits	
	PUSH	PSW
	CALL	PortInc			;ADDRESS MONTH REGISTER (8)
	POP	PSW
	OUT	CMOSPort+1		;(Quickly) STORE IT
	
	DCX	H
	MOV	A,M			;GET YEAR BYTE
	CALL	BINTOBCD		;Convert [A] to two BCD digits	
	PUSH	PSW		
	CALL	PortInc			;ADDRESS OF YEAR REGISTER (9)
	POP	PSW
	OUT	CMOSPort+1		;(Quickly) STORE IT

	MVI	E,31H			;POINT TO SAVE AREA
	CALL	PortInc		
	MVI	A,20H			;GET CENTURY BYTE (32)
	OUT	CMOSPort+1		;SAVE IT at 31H in CMOS RAM area

	POPIX
	POP	H
	POP	D
	POP	B
	RET



;--------- GET TIME & DATE ----------------------------------------------------------------------------------------					

;The BIOS needs to update SCB (@DATE, @ HOURS etc) getting data from RTC chip


GETTM:	CALL	UpdInPR			;BIOS should update the SCB (@DATE, @ HOURS etc) getting data from RTC
	JNC	RTC4A
	JP	TERROR			;ON ERROR
	
RTC4A:	MVI	E,-2			;-2 goes to 0 for PortInc2 
	CALL	PortInc2		;SET ADDRESS OF SECONDS (0)
	IN	CMOSPort+1		;Get BCD value returned
	STA	@SEC			;Store in SCB @SEC as BCD
	CALL	PortInc2		;SET ADDRESS OF MINUTES
	IN	CMOSPort+1		;Get BCD value returned
	STA	@MIN			;Store in SCB @MIN as BCD
	CALL	PortInc2		;SET ADDRESS OF HOURS
	IN	CMOSPort+1		;Get BCD value returned
	STA	@HOUR			;Store in SCB @HOUR as BCD
	
					;We now need to convert CMOS date to CPM total days since 1/1/1973 
	LXI	H,TBUFFER+2		;POINT TO TBUFFER
	MVI	E,6
	CALL	PortInc			;POINT TO DAY (7)
	IN	CMOSPort+1
	CALL	BCDTOBIN
	MOV	M,A			;SAVE Day Binary value in TBUFFER+2
	
	CALL	PortInc			;POINT TO MONTH (8)
	IN	CMOSPort+1
	DCX	H
	CALL	BCDTOBIN
	MOV	M,A			;SAVE Month Binary value in TBUFFER+1
	
	CALL	PortInc			;POINT TO YEAR (9)
	IN	CMOSPort+1
	DCX	H
	CALL	BCDTOBIN
	MOV	M,A			;SAVE Year Binary value in TBUFFER
	
					;Now have in TBUFFER Binary values of year, month & date
	LXIX	TBUFFER			;[A], [HL] & [IX] point to year
	LDX	A,YROFS
	CPI	100			;Past decimal Binary 99
	JNC	TERROR			;> 99 so error
	ANI	3			;Is current year a leap year
	MOV	B,A			;Store in in B
	MVI	A,28			;Feb has 28 days
	JNZ	TSKIP			;Jump if not
	INR	A			;Then set 29 for Feb
TSKIP:	STA	MFEB			;Fix Feb (store 28 or 29)
	LXI	H,1F63H			;Initiate Julian date count (starting from 1/1/1999)
					;Note DR starts with 0 at 1/1/1987. Counting from Y2000 is simplier/faster
					;BUT, you cannot set dates < 1/1/2000!
	LXI	D,MTBL			;Get month table pointer
	LDX	A,MNOFS			;Get current month
	DCR	A			;0 Relative
	JM	TERROR
	CPI	12
	JNC	TERROR		
	MOV	B,A			;Use it as a counter in B
	   
DSUM:	MOV	A,B			;Has count expired
	ORA	A
	JZ	DDAY
	DCR	B
	LDAX	D
	INX	D			;Next month
	CALL	ADDHL			;Add it in
	JMP	DSUM

DDAY:	LDX	A,DTOFS			;Get date
	DCR	A			;So carry works on compare
	JM	TERROR
	XCHG				;HL now = pointer to Month table
	CMP	M			;test date against current
	XCHG				;Restore Julian date pointer in HL
	JNC	TERROR			
	INR	A
	CALL	ADDHL			;Add it in
					;HL now has Julian date for current year.
					;Now add in days since 1/1/1978
	LDX	A,YROFS			;Get current year
	MOV	B,A			;Save for comparisons
	MVI	A,00			;Get base year (Y2000)
	MOV	C,A			;Save in C
	LXI	D,365
DLOP:	MOV	A,C
	CMP	B			;Are we up to current year
	JZ	BACK
	INR	C
	DAD	D			;Add in another year
	ANI	3			;Was it a leap year
	JNZ	DLOP
	INX	H
	JMP	DLOP
	
TERROR:	STC
BACK:	SHLD	@DATE			;Store word (binary) for CPM
	POPIX
	POP	H
	POP	D
	POP	B
	RET
	
	
	
;------------------ Time/CMOS RTC Support routines ------------------------------------
		
UpdInPR:				;Check we are ready to read clock
	PUSH	B
	LXI	B,600			;SET LOOP COUNT
UPDATE:	MVI	A,0AH			;ADDRESS OF [A] REGISTER
	OUT	CMOSPort
	NOP
	NOP
	NOP
	IN	CMOSPort+1		;READ  IN REGISTER [A]
	ANI	80H			;IF 8XH--> UIP BIT  IS ON (CANNOT READ TIME)
	JZ	UpdInPREND		;Are we ready/done
	DCX	B
	MOV	A,C
	ORA	B
	JNZ	UPDATE			;Try again
	XRA	A			
	SCF				;SET CARRY FOR ERROR
	POP	B
	RET
UpdInPREND:
	XRA	A			;Clear Carry
	POP	B
	RET				;RETURN
	
InitilizationStatus:	
	PUSH	D			;SAVE
	MVI	E,09H
	CALL	PortInc
	MVI	A,26H
	OUT	CMOSPort+1		;INITIALIZE 'A' REGISTER
	CALL	PortInc
	MVI	A,82H			;SET 'SET BIT' FOR CLOCK INITIALIZATION
					;AND 24 HOUR MODE
	OUT	CMOSPort+1		;INITIALIZE 'B' REGISTER
	CALL	PortInc
	IN	CMOSPort+1		;READ REGISTER 'C' TO INITIALIZE
	CALL	PortInc
	IN	CMOSPort+1		;READ REGISTER  'D'  TO INITIALIZE
	POP	D			;RESTORE
	RET

PortInc:
	MOV	A,E
	INR	A			;INCREMENT ADDRESS
	MOV	E,A
	OUT	CMOSPort
	RET


PortInc2:
	MOV	A,E
	ADI	2			;INCREMENT ADDRESS
	MOV	E,A
	OUT	CMOSPort
	RET
	

		
ADDHL:	ADD	L			;Add A to HL
	MOV	L,A
	RNC
	INR	H
	RET
	
BCDTOBIN:				;Convert BCD number (Two digits only) in A to Binary
	PUSH	B
	PUSH	D
	MOV	B,A			;Save it
	ANI	0FH			;Mask most significant four bits
	MOV	C,A			;Save unpacked BCDI in C register
	MOV	A,B			;Get BCD again
	ANI	0F0H			;Mask least significant four bits
	RRC				;Convert most significant four bits into unpacked BCD2
	RRC
	RRC
	RRC
	MOV	B,A			;Save unpacked BCD2 in B register
	XRA	A			;Clear accumulator (sum = 0)
	MVI	D,0AH			;Set D as a multiplier of 10
Sum:	ADD	D			;Add 10 until (B) = 0
	DCR	B			;Decrement BCD2 by one
	JNZ	Sum			;Is multiplication complete? i if not, go back and add again
	ADD	C			;Add BCD1
	POP	D
	POP	B
	RET				;Return with binary value in [A]
	

;Generalized Subroutine to convert binary number in [A] into its equivalent BCD number stored in RAM:
;Will return right most 2 digits in [A]

BINTOBCD:				;Convert binary number in [A] to BCD in RAM
       PUSH	B			;Save BC register pair contents
       PUSH	D			;Save DE register pair contents
       MVI	B,64H			;Load divisor decimal 100 in B register
       MVI	C,0AH			;Load divisor decimal 10 in C register
       MVI	D,00H			;Initialize Digit 1
       MVI	E,00H			;Initialize Digit 2
STEP1: CMP	B			;Check if number < Decimal 100
       JC	STEP2			;if yes go to step 2
       SUB	B                       ;Subtract decimal 100
       INR	E                       ;update quotient
       JMP	STEP1			;go to step 1
STEP2: CMP	C			;Check if number < Decimal 10
       JC	STEP3			;if yes go to step 3
       SUB	C                       ;Subtract decimal 10
       INR	D                       ;Update quotient
       JMP	STEP2			;Continue division by 10
STEP3: STA	Digit0			;Store Digit 0
       MOV	A,D			;Get Digit 1
       STA	Digit1			;Store Digit 1
       RLC				;Shift it up 4 bits
       RLC
       RLC
       RLC
       ANI	0F0H
       MOV	D,A			;Temp store in D
       MOV	A,E			;Get Digit 2
       STA	Digit2			;Store Digit 2
       LDA	Digit0
       ORA	D			;Add in 10's digit
       POP	D			;Restore DE register pair
       POP	B                       ;Restore BC register pair
       RET				;Return to main program	
       
      
Digit0:	DB	0			;Store for Bin->BCD
Digit1:	DB	0
Digit2:	DB	0      
       
MTBL:	DB	31			;Table of days/month for date lookup
MFEB:	DB	28
	DB	31,30,31,30
	DB	31,31,30,31,30,31

	IF	BANKED
	CSEG
	ENDIF

	;;;;;
	; CP/M BDOS FUNCTION INTERFACES

	; OPEN FILE:
OPEN:
	MVI C,15 ! JMP BDOS		; OPEN FILE CONTROL BLOCK

	; SET DMA ADDRESS:
SETDMA:
	MVI C,26 ! JMP BDOS		; SET DATA TRANSFER ADDRESS

	; SET MULTI SECTOR I/O COUNT:
SETMULTI:
	MVI C,44 ! JMP BDOS		; SET RECORD COUNT

	; READ FILE RECORD:
READ:
	MVI C,20 ! JMP BDOS		; READ RECORDS

	; CCP NOT FOUND ERROR MESSAGE:
CCP$MSG:
	DB	CR,LF,'BIOS ERR ON A: NO CCP.COM FILE',0

TBUFFER	DB	0,0,0,0			;Tempory buffer for calculating data

	; FCB FOR CCP.COM FILE LOADING:
CCP$FCB:
	DB	1		;AUTO-SELECT DRIVE A
	DB	'CCP     COM'	;FILE NAME AND TYPE
	DB	0,0,0,0
	DS	16
FCB$NR:	DB	0,0,0


	; SYSTEM SIGN-ON MESSAGE:
SIGNON$MSG:
	DB	CR,LF,LF,'CP/M VERSION 3.0 (Banked Version) (John Monahan 10/17/2021)'
	DB	CR,LF,'A: & B:=IDE/CF Cards. '
	DB	CR,LF,0
	END

	END
